/*
Copyright (c) 2016, The `code.ivysaur.me/makedeb` author(s)

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted, provided that the above copyright
notice and this permission notice appear in all copies.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF
THIS SOFTWARE.
*/

package makedeb

import (
	"archive/tar"
	"compress/gzip"
	"errors"
	"fmt"
	"io"
	"path"
	"strings"
	"time"
)

type tarGz struct {
	gzw          *gzip.Writer
	tw           *tar.Writer
	existingDirs map[string]struct{}
}

func NewTarGz(output io.Writer) (*tarGz, error) {

	gzw, err := gzip.NewWriterLevel(output, 9)
	if err != nil {
		return nil, err
	}

	tw := tar.NewWriter(gzw)

	return &tarGz{
		gzw:          gzw,
		tw:           tw,
		existingDirs: make(map[string]struct{}, 0),
	}, nil
}

func (this *tarGz) Close() {
	this.tw.Close()
	this.gzw.Close()
}

func (this *tarGz) AddContent(content string, TargetFileName string, FileMode int64, ModifyTime time.Time) error {
	return this.AddFile(strings.NewReader(content), int64(len(content)), TargetFileName, FileMode, ModifyTime)
}

// filemodeToDirmode converts an octal mode from e.g. 0644 into 0755, by adding
//  a matching execution flag for every read flag.
func filemodeToDirmode(FileMode int64) int64 {
	DirMode := FileMode | 040000 // d-flag in tar spec
	if FileMode|0400 > 0 {
		DirMode |= 0100
	}
	if FileMode|0040 > 0 {
		DirMode |= 0010
	}
	if FileMode|0004 > 0 {
		DirMode |= 0001
	}
	return DirMode
}

func (this *tarGz) AddDirectory(TargetDirectoryName string, DirMode int64, ModifyTime time.Time) error {

	if len(TargetDirectoryName) == 0 {
		return nil // nothing to do
	}

	// Assert TargetDirectoryName always ends with a trailing slash
	if TargetDirectoryName[len(TargetDirectoryName)-1] != byte('/') {
		TargetDirectoryName = TargetDirectoryName + "/"
	}

	// Skip if already added
	if _, check := this.existingDirs[TargetDirectoryName]; check {
		return nil
	}

	// Build parent directories first
	remainder, _ := path.Split(TargetDirectoryName[:len(TargetDirectoryName)-1])
	if len(remainder) > 1 {
		err := this.AddDirectory(remainder, DirMode, ModifyTime)
		if err != nil {
			return err
		}
	}

	// Add this directory
	err := this.tw.WriteHeader(&tar.Header{
		Name:     TargetDirectoryName,
		Mode:     DirMode,
		Size:     0,
		Typeflag: tar.TypeDir,
		ModTime:  ModifyTime,
	})
	if err != nil {
		return err
	}

	// Done
	this.existingDirs[TargetDirectoryName] = struct{}{}
	return nil
}

func (this *tarGz) AddFile(input io.Reader, InputSize int64, TargetFileName string, FileMode int64, ModifyTime time.Time) error {

	// Build necessary directory components first
	dirname, _ := path.Split(TargetFileName)
	err := this.AddDirectory(dirname, filemodeToDirmode(FileMode), ModifyTime)
	if err != nil {
		return err
	}

	err = this.tw.WriteHeader(&tar.Header{
		Name:     TargetFileName,
		Mode:     FileMode,
		Size:     InputSize,
		Typeflag: tar.TypeReg,
		ModTime:  ModifyTime,
	})
	if err != nil {
		return fmt.Errorf("couldn't add file '%s': %s", TargetFileName, err.Error())
	}

	n, err := io.Copy(this.tw, input)
	if err != nil {
		return err
	}

	if n != InputSize {
		return errors.New("Size mismatch")
	}

	return nil
}
