/*
Copyright (c) 2016, The `code.ivysaur.me/makedeb` author(s)

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted, provided that the above copyright
notice and this permission notice appear in all copies.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF
THIS SOFTWARE.
*/

package main

import (
	"errors"
	"flag"
	"fmt"
	"os"
	"path/filepath"
	"strings"

	"code.ivysaur.me/makedeb"
)

func sampleDirToDeb(SourceDir, packageName, Version string) (outputFileName string, err error) {

	sdir, err := os.Stat(SourceDir)
	if err != nil {
		return "", err
	}

	if !sdir.IsDir() {
		return "", errors.New("Target isn't a directory")
	}

	if len(packageName) == 0 {
		packageName = sdir.Name()
	}
	outputFileName = packageName + "-" + Version + ".deb"

	deb := makedeb.DebPackage{}
	deb.Attributes.Architecture = "all"
	deb.Attributes.Description = "Sample package"
	deb.Attributes.Maintainer = "Nobody <nobody@example.com>"
	deb.Attributes.Package = packageName
	deb.Attributes.Version = Version

	// Add sample scripts

	deb.Scripts = append(deb.Scripts, makedeb.InstallScript{
		Type: makedeb.INSTALLSCRIPT_PREINST,
		Content: `#!/bin/bash
echo "Inside preinst"
`,
	})

	deb.Scripts = append(deb.Scripts, makedeb.InstallScript{
		Type: makedeb.INSTALLSCRIPT_POSTINST,
		Content: `#!/bin/bash
echo "Inside postinst"
`,
	})

	// Add content files

	filepath.Walk(SourceDir, func(fpath string, stat os.FileInfo, err error) error {
		if stat.IsDir() {
			return nil
		}

		deb.Files.AddFile(
			fpath, // source
			strings.Replace(filepath.Clean("opt/makedeb/"+packageName+"/"+fpath), `\`, `/`, -1), // target - always use forwardslashes for linux
			int64(int(stat.Mode())),
			stat.ModTime(),
		)
		return nil
	})

	// Write out .deb content

	of, err := os.OpenFile(outputFileName, os.O_CREATE|os.O_WRONLY, 0644)
	if err != nil {
		return "", err
	}
	defer of.Close()

	return outputFileName, deb.Generate(of)
}

func main() {
	SourceDir := flag.String("SourceDir", ".", "Read file content from this directory")
	PackageName := flag.String("PackageName", "", "Set package name (optional)")
	Version := flag.String("Version", "0.0.0", "Set package version")

	flag.Parse()

	outFile, err := sampleDirToDeb(*SourceDir, *PackageName, *Version)
	if err != nil {
		fmt.Fprintf(os.Stderr, "Error: %s\n", err.Error())
		os.Exit(1)
	}

	fmt.Printf("Written to '%s'\n", outFile)
	os.Exit(0)
}
